import os

from copy import deepcopy
from xepmts.api.secrets import MONGO_PASSWORD
from xepmts.api.domain import get_domain

DOMAIN = get_domain()

URL_PREFIX = os.getenv("XEPMTS_URL_PREFIX", "")
API_VERSION = "v1"
RESOURCE_METHODS = ["GET", "POST"]
ITEM_METHODS = ["GET", "PUT", "PATCH", "DELETE"]
ALLOWED_READ_ROLES = ["admin", "superuser", "expert", "user", "read", "write"]
ALLOWED_WRITE_ROLES = ["admin", "superuser", "expert", "write"]
EMBEDDING = True
MEDIA_PATH = "files"
PAGINATION_LIMIT = 10000
SCHEMA_ENDPOINT = "schema"
IF_MATCH = True
ENFORCE_IF_MATCH = False
HATEOAS = True
VERSIONS = "_versions"
NORMALIZE_ON_PATCH = False

# ----------------- Mongo config ------------------------------------------ #
MONGO_HOST = os.getenv("XEPMTS_MONGO_HOST", "localhost")
MONGO_PORT = int(os.getenv("XEPMTS_MONGO_PORT", 27017))
MONGO_DBNAME = os.getenv("XEPMTS_MONGO_DB", "pmts")
MONGO_USERNAME = os.getenv("XEPMTS_MONGO_USER", "")
MONGO_AUTH_SOURCE = os.getenv("XEPMTS_MONGO_AUTH_SOURCE", MONGO_DBNAME)

MONGO1T_HOST = MONGO_HOST
MONGO1T_PORT = MONGO_PORT
MONGO1T_DBNAME = MONGO_DBNAME + "1t"
MONGO1T_AUTH_SOURCE = MONGO_AUTH_SOURCE
MONGO1T_PASSWORD = MONGO_PASSWORD
if os.getenv("XEPMTS_MONGO_REPLICA_SET", ""):
    MONGO_REPLICA_SET = os.getenv("XEPMTS_MONGO_REPLICA_SET", "")
    MONGO1T_REPLICA_SET = MONGO_REPLICA_SET

MONGO1T_USERNAME = MONGO_USERNAME

if os.getenv("XEPMTS_MONGO_URI", ""):
    MONGO_URI = os.getenv("XEPMTS_MONGO_URI", "")
    MONGO1T_URI = MONGO_URI
# -------------------------------------------------------------------------- #

SERVERS = ["https://api."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),]

X_DOMAINS = ['http://localhost:8000',
            'http://127.0.0.1:8000',
            'http://127.0.0.1:5000',
            'http://editor.swagger.io',
            "https://"+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
            "https://api."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
            "https://website."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
            "https://panels."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
            "https://catalog."+os.getenv('XEPMTS_DOMAIN','pmts.xenonnt.org'),
             ]

X_HEADERS = ['Content-Type', 'If-Match', 'Authorization', 'X-HTTP-Method-Override']  # Needed for the "Try it out" buttons

def get_settings_dict(**overrides):
    
    domain_overrides = overrides.pop("DOMAIN", {})
    domain = dict(DOMAIN)
    domain.update(domain_overrides)

    settings = dict(
        DOMAIN = domain,
        URL_PREFIX = URL_PREFIX,
        API_VERSION = API_VERSION,
        RESOURCE_METHODS = RESOURCE_METHODS,
        ITEM_METHODS = ITEM_METHODS,
        ALLOWED_READ_ROLES = ALLOWED_READ_ROLES,
        ALLOWED_WRITE_ROLES = ALLOWED_WRITE_ROLES,
        EMBEDDING = EMBEDDING,
        MEDIA_PATH = MEDIA_PATH,
        PAGINATION_LIMIT = PAGINATION_LIMIT,
        SCHEMA_ENDPOINT = SCHEMA_ENDPOINT,
        IF_MATCH = IF_MATCH,
        ENFORCE_IF_MATCH = ENFORCE_IF_MATCH,
        HATEOAS = HATEOAS,
        VERSIONS = VERSIONS,
        NORMALIZE_ON_PATCH = NORMALIZE_ON_PATCH,
        
        MONGO1T_HOST = MONGO1T_HOST,
        MONGO1T_PORT = MONGO1T_PORT,
        MONGO1T_DBNAME = MONGO1T_DBNAME,
        MONGO1T_USERNAME = MONGO1T_USERNAME,
        MONGO1T_PASSWORD = MONGO1T_PASSWORD,
        MONGO1T_AUTH_SOURCE = MONGO1T_AUTH_SOURCE,
        
        MONGO_HOST = MONGO_HOST,
        MONGO_PORT = MONGO_PORT,
        MONGO_DBNAME = MONGO_DBNAME,
        MONGO_USERNAME = MONGO_USERNAME,
        MONGO_PASSWORD = MONGO_PASSWORD,
        MONGO_AUTH_SOURCE = MONGO_AUTH_SOURCE,

        SERVERS = SERVERS,
        X_DOMAINS = X_DOMAINS,
        X_HEADERS = X_HEADERS,
    )
    if os.getenv("XEPMTS_MONGO_URI", ""):
        settings["MONGO1T_URI"] = MONGO1T_URI
        settings["MONGO_URI"] = MONGO_URI

    if os.getenv("XEPMTS_MONGO_REPLICA_SET", ""):
        settings["MONGO_REPLICA_SET"] = MONGO_REPLICA_SET
        settings["MONGO1T_REPLICA_SET"] = MONGO1T_REPLICA_SET

    settings.update(overrides)
    return settings


"""Console script for xepmts."""
import sys
import os

import click
from xepmts.api.utils import resources_from_templates, read_endpoint_files
from xepmts.api.domain import DEFAULT_TEMPLATE_DIR
from ruamel.yaml import YAML
yaml = YAML()
yaml.indent(mapping=4, sequence=4, offset=2)


@click.group()
def main():
    """Console script for xepmts."""
    return 0

@main.command()
@click.option('--template_dir', default=DEFAULT_TEMPLATE_DIR, help='Template directory')
@click.option('--out', default="./api_server/endpoints", help='Output directory')
def generate_endpoints(template_dir, out):
    # import eve
    if not os.path.isdir(out):
        os.makedirs(out)
    templates = read_endpoint_files(template_dir)
    domain = resources_from_templates(templates)
    for k, v in domain.items():
        if "url" in v:
            rpath, _, fname = v["url"].rpartition("/")
            abspath = os.path.join(out, rpath)
            if not os.path.exists(abspath):
                os.makedirs(abspath)
        else:
            fname = k
            abspath = out
        fpath = os.path.join(abspath, fname+".yml")
        endpoint = {k: v}
        with open(fpath, "w") as f:
            yaml.dump(endpoint, f)

@main.command()
def serve():
    from xepmts.api.app import make_local_app
    app = make_local_app()
    app.run(host="localhost", debug=True, ) #ssl_context="adhoc"


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover




import os
from xepmts.api.utils import read_endpoint_files, resources_from_templates


SOURCE_DIR = os.path.dirname(os.path.abspath(__file__))
DEFAULT_TEMPLATE_DIR = os.path.join(SOURCE_DIR, "endpoint_templates")

def get_domain():
    
    ENDPOINT_TEMPLATE_DIR = os.getenv("XEPMTS_ENDPOINT_TEMPLATE_DIR", DEFAULT_TEMPLATE_DIR)
    ENDPOINT_DIR = os.getenv("XEPMTS_ENDPOINT_DIR", "")
    if not ENDPOINT_DIR:
        templates = read_endpoint_files(ENDPOINT_TEMPLATE_DIR)
        DOMAIN = resources_from_templates(templates)
    else:
        DOMAIN = read_endpoint_files(ENDPOINT_DIR)
    return DOMAIN


"""Top-level package for xepmts."""

__author__ = """Yossi Mosbacher"""
__email__ = 'joe.mosbacher@gmail.com'
__version__ = '0.4.2'

def settings(**kwargs):
    from eve_panel import settings as panel_settings
    if not kwargs:
        return dir(panel_settings)
    else:
        for k,v in kwargs.items():
            setattr(panel_settings, k, v)

def default_client():
    from xepmts.api.client import default_client
    return default_client()

def notebook():
    import panel as pn
    return pn.extension()




# -*- coding: utf-8 -*-
import os

from xepmts.api.settings import get_settings_dict


def make_app(settings=None, auth=None, app=None,
             swagger=True, swagger_ui=True, fs_store=False,
             export_metrics=True):
    if app is None and settings is None:
        settings = get_settings_dict()
        raise RuntimeError("App or settings need to be defined.")
    kwargs = {}
    if fs_store:
        from eve_fsmediastorage import FileSystemMediaStorage
        kwargs["media"] = FileSystemMediaStorage
    if app is None:
        from eve import Eve
        app = Eve(settings=settings, auth=auth, **kwargs)

    if swagger:
        # from eve_swagger import swagger as swagger_blueprint
        from eve_swagger import get_swagger_blueprint
        
        swagger_blueprint = get_swagger_blueprint()
        app.register_blueprint(swagger_blueprint)
        app.config['SWAGGER_INFO'] = {
            'title': 'XENON PMT API',
            'version': '1.0',
            'description': 'API for the XENON PMT database',
            'termsOfService': 'https://opensource.org/ToS',
            'contact': {
                'name': 'Yossi Mosbacher',
                'url': 'https://pmts.xenonnt.org',
                "email": "joe.mosbacher@gmail.com"
            },

            'license': {
                'name': 'BSD',
                'url': 'https://github.com/nicolaiarocci/eve-swagger/blob/master/LICENSE',
            
            },
            'schemes': ['http', 'https'],

        }

    if swagger_ui:
        from flask_swagger_ui import get_swaggerui_blueprint

        API_URL = '/api-docs'
        SWAGGER_URL = ''
        SWAGGERUI_BLUEPRINT = get_swaggerui_blueprint(
            SWAGGER_URL,
            API_URL,
            config={
                'app_name': "PMT Database API"
            },
        )
        app.register_blueprint(SWAGGERUI_BLUEPRINT, url_prefix=SWAGGER_URL)
    if export_metrics:
        from prometheus_flask_exporter import PrometheusMetrics
        PrometheusMetrics(app)
    return app


def make_local_app():
    import eve
    settings = get_settings_dict()
    app = eve.Eve(settings=settings)
    return app
    


import eve_panel
from xepmts.api import app
import os
import pkg_resources

APPS = {
    entry_point.name: entry_point.load()
    for entry_point
    in pkg_resources.iter_entry_points('xepmts.apps')
}

def make_client(app_names=None):
    if app_names is None:
        app_names = list(APPS)
    apps = {name: APPS[name]() for name in app_names}
    
    client = eve_panel.EveClient.from_apps_dict(apps, name="xepmts", sort_by_url=True)
    for name in apps:
        getattr(client, name)._http_client.auth.set_auth_by_name("Bearer")
    return client

def default_client():
    return make_client()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tests', 'xepmts', 'xepmts.api']

package_data = \
{'': ['*'], 'xepmts.api': ['endpoint_templates/*']}

install_requires = \
['click', 'eve_panel']

entry_points = \
{'console_scripts': ['xepmts = xepmts.cli:main'],
 'xepmts.apps': ['db = xepmts.api.app:make_local_app']}

setup_kwargs = {
    'name': 'xepmts',
    'version': '0.4.2',
    'description': 'Top-level package for xepmts.',
    'long_description': '======\nxepmts\n======\n\n\n.. image:: https://img.shields.io/pypi/v/xepmts.svg\n        :target: https://pypi.python.org/pypi/xepmts\n\n.. image:: https://img.shields.io/travis/jmosbacher/xepmts.svg\n        :target: https://travis-ci.com/jmosbacher/xepmts\n\n.. image:: https://readthedocs.org/projects/xepmts/badge/?version=latest\n        :target: https://xepmts.readthedocs.io/en/latest/?badge=latest\n        :alt: Documentation Status\n\nBasic usage::\n\n        import xepmts\n\n        # If you are using a notebook:\n        xepmts.notebook()\n\n        db = xepmts.default_client().db\n        db.set_token(\'YOUR-API-TOKEN\')\n\n        # set the number of items to pull per page\n        db.tpc.installs.items_per_page = 25\n\n        # get the next page \n        page = db.tpc.installs.next_page()\n\n        # iterate over pages:\n        for page in db.tpc.installs.pages():\n                df = page.df\n                # do something with data\n\n        # select only top array\n        top_array = db.tpc.installs.filter(array="top")\n        \n        # iterate over top array pages\n        for page in top_array.pages():\n                df = page.df\n                # do something with data\n\n        query = dict(pmt_index=4)\n        # get the first page of results for this query as a list of dictionaries\n        docs = db.tpc.installs.find(query, max_results=25, page_number=1)\n\n        # same as find, but returns a dataframe \n        df = db.tpc.installs.find_df(query)\n\n\n        # insert documents into the database\n        docs = [{"pmt_index": 1, "position_x": 0, "position_y": 0}]\n        db.tpc.installs.insert_documents(docs)\n    \n\n* Free software: MIT\n* Documentation: https://xepmts.readthedocs.io/\n\n\nFeatures\n--------\n\n* TODO\n\nCredits\n-------\n\nThis package was created with Cookiecutter_ and the `briggySmalls/cookiecutter-pypackage`_ project template.\n\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _`briggySmalls/cookiecutter-pypackage`: https://github.com/briggySmalls/cookiecutter-pypackage\n',
    'author': 'Yossi Mosbacher',
    'author_email': 'joe.mosbacher@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jmosbacher/xepmts',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)
